/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

void CalibrateDlg::init()
{
  m_item = 0;
  m_sample = new QPixmap(100, 100);
  fillSchemeList();
  fillColorList();
}

void CalibrateDlg::destroy()
{
  delete m_sample;
}

void CalibrateDlg::fillSchemeList()
{
  SchemeList->insertStringList(static_cast<KXStitchApplication*>(kapp)->schemes());
}

void CalibrateDlg::fillColorList()
{
  ColorList->clear();
  m_scheme = SchemeList->currentText();
  FlossScheme *scheme = static_cast<KXStitchApplication*>(kapp)->scheme(m_scheme);
  QPtrListIterator<Floss> it = scheme->flossIterator();
  Floss* f;
  for ( ; (f = it.current()) ; ++it)
    new FlossListBoxItem(ColorList, f);
  ColorList_selectionChanged(ColorList->item(0));
}

void CalibrateDlg::updateSample()
{
  m_sample->resize(ExampleColor->size());
  m_sample->fill(m_sampleColor);
  ExampleColor->setPixmap(*m_sample);
  RedSlider->setValue(m_sampleColor.red());
  GreenSlider->setValue(m_sampleColor.green());
  BlueSlider->setValue(m_sampleColor.blue());
}

void CalibrateDlg::updateName(bool modified)
{
  SelectedColorName->setText(QString("%1-%2%3").arg(m_item->floss->name).arg(m_item->floss->description).arg((modified)?i18n(" (Modified)"):""));
}

void CalibrateDlg::commitColor()
{
  if (m_item->floss->color != m_sampleColor)
  {
    m_calibratedColors[m_scheme][m_item->floss->name] = m_sampleColor;
    m_item->setCalibrated(true);
  }
}

void CalibrateDlg::OKButton_clicked()
{
  commitColor();

  QMap<QString, ChangedColors>::Iterator it;
  for (it = m_calibratedColors.begin() ; it != m_calibratedColors.end() ; ++it)
  {
    FlossScheme* scheme = static_cast<KXStitchApplication*>(kapp)->scheme(it.key());
    QPtrListIterator<Floss> itFloss = scheme->flossIterator();
    for (Floss* f = 0 ; (f = itFloss.current()) ; ++itFloss)
      if (it.data().contains(f->name))
        f->color = it.data()[f->name];
    static_cast<KXStitchApplication*>(kapp)->writeScheme(it.key());
  }
  QDialog::accept();
}

void CalibrateDlg::HelpButton_clicked()
{
  kapp->invokeHelp("CalibrateDialog");
}

void CalibrateDlg::SchemeList_activated(const QString&)
{
  commitColor();
  fillColorList();
}

void CalibrateDlg::ColorList_selectionChanged(QListBoxItem *item)
{
  if (m_item)
    commitColor();
  m_item = static_cast<FlossListBoxItem*>(item);
  if (m_calibratedColors[m_scheme].contains(m_item->floss->name))
    m_sampleColor = m_calibratedColors[m_scheme][m_item->floss->name];
  else
    m_sampleColor = m_item->floss->color;
  updateSample();
  updateName(m_calibratedColors[m_scheme].contains(m_item->floss->name));
}

void CalibrateDlg::RedSlider_valueChanged(int red)
{
  m_sampleColor.setRgb(red, m_sampleColor.green(), m_sampleColor.blue());
  updateSample();
  updateName(true);
}

void CalibrateDlg::GreenSlider_valueChanged(int green)
{
  m_sampleColor.setRgb(m_sampleColor.red(), green, m_sampleColor.blue());
  updateSample();
  updateName(true);
}

void CalibrateDlg::BlueSlider_valueChanged(int blue)
{
  m_sampleColor.setRgb(m_sampleColor.red(), m_sampleColor.green(), blue);
  updateSample();
  updateName(true);
}

void CalibrateDlg::ResetColor_clicked()
{
  m_sampleColor = m_item->floss->color;
  if (m_calibratedColors[m_scheme].contains(m_item->floss->name))
    m_calibratedColors[m_scheme].remove(m_item->floss->name);
  updateSample();
  updateName(false);
  m_item->setCalibrated(false);
}
